!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Returns the magnetic field field line equations, and the tangent map if required.

!latex \item Sometimes it is required to have the full contravariant components of the field, so the quantity \verb+Bzeta+$\equiv B^\zeta$ is returned through memory.

!latex \end{enumerate} \subsubsection{equations of field line flow} \begin{enumerate}

!latex \item The field line equations are $\dot\theta=B^\theta/B^\zeta$, $\dot s=B^s/B^\zeta$.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine bf00aa( arbitrarylabel, st, Bst ) ! the format of this subroutine is constrained by the NAG ode integration routines. 

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  use constants, only : zero, pi2, pi

  use numerical, only : vsmall

  use fileunits, only : ounit

  use inputlist, only : Wbf00aa, Nvol, Lpoincare

  use cputiming, only : Tbf00aa

  use allglobal, only : myid, cpus, mn, im, in, ivol, Ltangent, interfacelabel, Bzeta, doAtmn, doAzmn

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  REAL, intent(in)  :: arbitrarylabel, st(8)
  REAL, intent(out) :: Bst(8)

  LOGICAL           :: Loutofdomain
  INTEGER           :: imn, ij, mm, ip, il, lvol, Ldo
  REAL              :: lss, teta, zeta, arg, carg, sarg, dBu(3,0:3), TM(2,2)

  BEGIN(bf00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  lvol = ivol ! the argument list of bf00aa is fixed by NAG requirements, but volume index is required by sa00aa below;

  FATALMESS(bf00aa,lvol.lt.1 .or. lvol.gt.Nvol, invalid volume: perhaps ivol has not been set)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( st(1).gt.interfacelabel(lvol) ) then ; Bst(1:6) = (/ zero , zero , zero , zero , zero , zero /) ; goto 9999 ! out of domain; should speed things up . . . ;
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Lpoincare.eq.0 ) then ; teta = st(2) ; zeta = arbitrarylabel
  endif
  if( Lpoincare.eq.1 ) then ; zeta = st(2) ; teta = arbitrarylabel
  endif
  if( Lpoincare.eq.2 ) then ; teta = st(2) ; zeta = st(7) ! zeta is one of our ODEs
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  lss = st(1) ! internal variable;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  ! If this is the innermost volume and s < 0, then just reverse its sign and send theta to theta + pi.
  ! Strictly we shouldn't be able to cross to s < 0, but this is possible due to finite step sizes,
  ! and we don't want to crash in this case.

  if( lvol.eq.1 .and. lss < 0 ) then
      lss = -lss
      teta = teta + pi
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  lss = min( max(lss,interfacelabel(lvol-1)), interfacelabel(lvol) ) ! local variable used to ensure point is within bounds;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{enumerate} \subsubsection{summation of Fourier harmonics} \begin{enumerate}

!latex \item A slow transform is performed, as the position $(s,\theta,\zeta)$ at which the field is required is arbitrary.
!latex       It may be possible to speed this up by re-organizing the loops.
  
  dBu(1:3,0:3) = zero ! initialize summation;

  Ldo = 0 
  WCALL(bf00aa,sa00aa,(lvol, lss, Ldo)) ! interpolate Fourier harmonics of vector potential;

  do imn = 1, mn

!  if( imask(imn).le.0 ) cycle

   arg = im(imn)*teta - in(imn)*zeta ; carg = cos(arg) ; sarg = sin(arg) ! sum Fourier modes;
   
   dBu(1,0) = dBu(1,0) + ( - im(imn)        * doAzmn(0,0,imn) - in(imn)          * doAtmn(0,0,imn) ) * sarg
   dBu(2,0) = dBu(2,0) - (                    doAzmn(0,1,imn)                                      ) * carg
   dBu(3,0) = dBu(3,0) + (                                                         doAtmn(0,1,imn) ) * carg

   if( Ltangent.eq.1 ) then

    dBu(1,1) = dBu(1,1) + ( - im(imn)        * doAzmn(0,1,imn) - in(imn)          * doAtmn(0,1,imn) ) * sarg
    dBu(2,1) = dBu(2,1) - (                    doAzmn(0,2,imn)                                      ) * carg
    dBu(3,1) = dBu(3,1) + (                                                         doAtmn(0,2,imn) ) * carg
    
    dBu(1,2) = dBu(1,2) + ( - im(imn)*im(imn) * doAzmn(0,0,imn) - in(imn)*im(imn) * doAtmn(0,0,imn) ) * carg
    dBu(2,2) = dBu(2,2) - ( -         im(imn) * doAzmn(0,1,imn)                                     ) * sarg
    dBu(3,2) = dBu(3,2) + (                                     -         im(imn) * doAtmn(0,1,imn) ) * sarg
    
    dBu(1,3) = dBu(1,3) + ( + im(imn)*in(imn) * doAzmn(0,0,imn) + in(imn)*in(imn) * doAtmn(0,0,imn) ) * carg ! need to check this;
    dBu(2,3) = dBu(2,3) - ( +         in(imn) * doAzmn(0,1,imn)                                     ) * sarg ! need to check this;
    dBu(3,3) = dBu(3,3) + (                                     +         in(imn) * doAtmn(0,1,imn) ) * sarg ! need to check this;
    
   endif ! end of if( Ltangent.eq.1 ) then;
   
  enddo ! end of do imn = 1, mn;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Lpoincare.eq.0 .and. abs(dBu(3,0)).lt.vsmall ) then
   cput = MPI_WTIME() ; write(ounit,'("bf00aa : "f10.2" : lvol="i3" ; zeta="es23.15" ; (s,t)=("es23.15" ,"es23.15" ) ; B^z="es23.15" ;")')cput-cpus,lvol,zeta,st(1:2),dBu(3,0)
   FATAL(bf00aa,abs(dBu(3,0)).lt.vsmall)
  endif
  
  if( Lpoincare.eq.1 .and. abs(dBu(2,0)).lt.vsmall ) then
   cput = MPI_WTIME() ; write(ounit,'("bf00aa : "f10.2" : lvol="i3" ; teta="es23.15" ; (s,z)=("es23.15" ,"es23.15" ) ; B^t="es23.15" ;")')cput-cpus,lvol,teta,st(1:2),dBu(2,0)
   FATAL(bf00aa,abs(dBu(2,0)).lt.vsmall)
  endif

  Bzeta = dBu(3,0) ! sometimes bf00aa is used to compute the field;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{enumerate} \subsubsection{normalization of field; definition of field line parameter} \begin{enumerate}

!latex \item The choice of integration parameter is controlled by \verb+Lpoincare+ as follows:

  select case( Lpoincare ) !latex \begin{itemize}

  case( 0 ) !latex \item \verb+Lpoincare=0+ : normalize to toroidal field, $B^\zeta$; the toroidal angle, $\zeta$,  is the integration parameter;

   Bst(1:2) = (/ dBu(1,0), dBu(2,0) /) / dBu(3,0)

  case( 1 ) !latex \item \verb+Lpoincare=1+ : normalize to poloidal field, $B^\theta$; the poloidal angle, $\theta$, is the integration parameter;
   
   Bst(1:2) = (/ dBu(1,0), dBu(3,0) /) / dBu(2,0)

  case( 2 ) !latex \item \verb+Lpoincare=2+ : not normalized;

      Bst(1:2) = (/ dBu(1,0), dBu(2,0) /)
      Bst(7) = dBu(3,0)
      Bst(8) = Bst(7) ! We have the same derivative here, but we've started from zero so that we can tell when we've gone 2pi/Nfp/Nz from where we started.

  case default

   FATALMESS(bf00aa,.true.,invalid Lpoincare) 

  end select !latex \end{itemize}
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \end{enumerate} \subsubsection{calculation of magnetic field outside computational domain} \begin{enumerate}

!latex \item If there is a non-zero normal field applied at the outermost interface, the field line may leave the computational domain.

  if( st(1).ge.interfacelabel(lvol) ) then ! outside computational domain;
   if( Bst(1).gt.zero ) Bst(1) = zero ! no need to go wandering off; stick close to computational boundary; go back inside where possible;
   Bst(3:6) = zero ; goto 9999 ! tangent map is not required;
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{enumerate} \subsubsection{definition of tangent map} \begin{enumerate}

!latex \item The tangent map satisfies 
!latex       \be \frac{d}{d \z} M = 
!latex       \left(\begin{array}{cc} \partial_\s \dot \s & \partial_\t \dot \s \\
!latex                               \partial_\s \dot \t & \partial_\t \dot \t 
!latex               \end{array}
!latex       \right)
!latex       M. \ee
!latex \item Note that the tangent map is only constructed if \verb+Ltangent.eq.1+.

  if( Ltangent.eq.0 ) then ; Bst(3:6) = zero ; goto 9999 ! tangent map is not required;
  endif
 
  select case( Lpoincare )
   
  case(0)
   
   TM(1,1) = ( dBu(1,1) - Bst(1) * dBu(3,1) ) / dBu(3,0) !  radial  derivative of  psi  dot;
   TM(1,2) = ( dBu(1,2) - Bst(1) * dBu(3,2) ) / dBu(3,0) ! poloidal derivative of  psi  dot;
   TM(2,1) = ( dBu(2,1) - Bst(2) * dBu(3,1) ) / dBu(3,0) !  radial  derivative of theta dot;
   TM(2,2) = ( dBu(2,2) - Bst(2) * dBu(3,2) ) / dBu(3,0) ! poloidal derivative of theta dot;
   
  case(1)
   
   TM(1,1) = ( dBu(1,1) - Bst(1) * dBu(2,1) ) / dBu(2,0) !  radial  derivative of  psi  dot;
   TM(1,2) = ( dBu(1,3) - Bst(1) * dBu(2,3) ) / dBu(2,0) ! poloidal derivative of  psi  dot;
   TM(2,1) = ( dBu(3,1) - Bst(2) * dBu(2,1) ) / dBu(2,0) !  radial  derivative of theta dot;
   TM(2,2) = ( dBu(3,3) - Bst(2) * dBu(2,3) ) / dBu(2,0) ! poloidal derivative of theta dot;
   
  case default
   
   FATALMESS(bf00aa,.true.,the tangent map has not been implemented for Lpoincare = 2) 
   
  end select
  
  Bst(3) = TM(1,1) * st(3) + TM(1,2) * st(5) ! tangent map obtained by matrix multiplication;
  Bst(4) = TM(1,1) * st(4) + TM(1,2) * st(6)
  Bst(5) = TM(2,1) * st(3) + TM(2,2) * st(5)
  Bst(6) = TM(2,1) * st(4) + TM(2,2) * st(6)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(bf00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine bf00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine bf00aa_out( arbitrarylabel, st )

  use constants, only : pi2
  use inputlist, only : Lpoincare
  use fileunits, only : ounit
  use allglobal, only : pi2nfp, Lfb!, iqfmin, jgd, kgd

  LOCALS

  REAL, intent(inout) :: arbitrarylabel
  REAL, intent(in)    :: st(8)
  
  if( Lpoincare.eq.0 ) arbitrarylabel = arbitrarylabel + pi2nfp ! next intermediate output location;
  if( Lpoincare.eq.1 ) arbitrarylabel = arbitrarylabel + pi2    ! next intermediate output location;
  if( Lpoincare.eq.2 ) arbitrarylabel = arbitrarylabel + pi2nfp ! next intermediate output location;

  return

end subroutine bf00aa_out

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

REAL function bf00aa_end( arbitrarylabel, st )

  use constants, only : pi2
  use inputlist, only : Lpoincare
  use allglobal, only : pi2nfp, Nz

  LOCALS

  REAL, intent(inout) :: arbitrarylabel
  REAL, intent(in)    :: st(8)
  
  if( Lpoincare.eq.0 ) bf00aa_end = arbitrarylabel   - pi2nfp    ! integration termination;
  if( Lpoincare.eq.1 ) bf00aa_end = arbitrarylabel   - pi2       ! integration termination;
  if( Lpoincare.eq.2 ) bf00aa_end = abs(st(8)) - pi2nfp/Nz ! integration termination;

  return

end FUNCTION bf00aa_end

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
