!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Describes some of the numerical tasks required to compute a stepped-pressure equilibrium \cite{SPEC} solution.

!latex \end{enumerate}

!latex \subsection{introduction}

!latex \begin{enumerate}

!latex \item Consider a toroidal volume consisting of nested subvolumes.
!latex In each subvolume it is required to compute the magnetic field (as described below).
!latex \item Then, it will be required to adjust the boundary of each subvolume in order to achieve global force balance (this problem is not considered here).

!latex \end{enumerate}

!latex \subsection{solving the Beltrami equation in a toroid} \label{sec:BeltramiTorus}

!latex \begin{enumerate}

!latex \item Assume that a smooth toroidal surface, ${\bf x}(\t,\z)$, is given.
!latex \item Compute the field that satisfies
!latex \be \nabla \times {\bf B} = \mu {\bf B}, \label{eq:Beltrami}
!latex \ee
!latex for a given $\mu$, in the toroidal volume ${\cal V}$ with boundary $\partial {\cal V}\equiv {\bf x}$.

!latex \item The field is tangential to the boundary, ${\bf B}\cdot {\bf n} = 0$.
!latex \item The enclosed toroidal flux is given, $\int_{\cal S} {\bf B}\cdot d{\bf s} = \psi_t$.

!latex \end{enumerate}

!latex \subsection{solving the Beltrami equation in a toroidal annulus} \label{sec:BeltramiAnnulus}

!latex \begin{enumerate}

!latex \item Assume that smooth toroidal surfaces, ${\bf x}_{inn}(\t,\z)$ and ${\bf x}_{out}(\t,\z)$, are given, and that ${\bf x}_{inn}$ is inside ${\bf x}_{out}$.
!latex \item Compute the field that satisfies
!latex \be \nabla \times {\bf B} = \mu {\bf B},
!latex \ee
!latex for a given $\mu$, in the toroidal volume ${\cal V}$
!latex with inner boundary $\partial {\cal V}\equiv {\bf x}_{inn}$ and outer boundary boundary $\partial {\cal V}\equiv {\bf x}_{out}$.

!latex \item The field is tangential to both boundaries, ${\bf B}\cdot {\bf n} = 0$.
!latex \item The enclosed toroidal flux is given, $\int_{\cal S} {\bf B}\cdot d{\bf s} = \psi_t$.
!latex \item The enclosed poloidal flux is given, $\int_{\cal S} {\bf B}\cdot d{\bf s} = \psi_p$.

!latex \end{enumerate}

!latex \subsection{solving Laplace's equation in a toroidal annulus} \label{sec:LaplaceAnnulus}

!latex \begin{enumerate}

!latex \item Assume that smooth toroidal surfaces, ${\bf x}_{inn}(\t,\z)$ and ${\bf x}_{out}(\t,\z)$, are given, and that ${\bf x}_{inn}$ is inside ${\bf x}_{out}$.
!latex \item Compute the field that satisfies
!latex \be \nabla \times {\bf B} = 0,
!latex \ee
!latex in the toroidal volume ${\cal V}$
!latex with inner boundary $\partial {\cal V}\equiv {\bf x}_{inn}$ and outer boundary boundary $\partial {\cal V}\equiv {\bf x}_{out}$.

!latex \item The field is tangential to the inner boundary, ${\bf B}\cdot {\bf n} = 0$.
!latex \item The field is {\em not} tangential to the outer boundary but instead the quantity ${\bf B}\cdot {\bf n}=b(\t,\z)$ is assumed given.
!latex \item The current linking the toroidal annulus is given, $\int_{\cal S} {\bf j}\cdot d{\bf s} = I$.
!latex \item The current inside the inner interface is given, $\int_{\cal S} {\bf j}\cdot d{\bf s} = G$.

!latex \end{enumerate}

!latex \subsection{required output}

!latex For each of the problems described in \Sec{BeltramiTorus},  \Sec{BeltramiAnnulus} and  \Sec{LaplaceAnnulus}, . . .

!latex \begin{enumerate}
!latex \item Usually, during the iterative scheme required to solve for the non-linear plasma equilibrium (which is not discussed here),
!latex the only place that ${\bf B}$ is required is on the boundary.
!latex It is only on completion (for post-diagnostics) is ${\bf B}$ required on the internal volume.

!latex \item However, it is sometimes required to compute the following integrals:
!latex \be {\rm magnetic \;\; energy}   & \equiv & \int_{\cal V} {\bf B}\cdot{\bf B} \, dv, \\
!latex     {\rm magnetic \;\; helicity} & \equiv & \int_{\cal V} {\bf A}\cdot{\bf B} \, dv
!latex \ee
!latex where ${\bf A}$ is the magnetic vector potential, ${\bf B}=\nabla \times {\bf A}$.
!latex In this case, it is required to compute the field, ${\bf B}$, that minimizes the magnetic energy subject to the constraint of conserved magnetic helicity.

!latex \item In either case, it is required to know how the required output
!latex (either ${\bf B}$ on the boundary, or $\int {\bf B}\cdot{\bf B} dv$ and $\int {\bf A}\cdot{\bf B}dv$)
!latex changes when either (i) $\mu$, (ii) the enclosed poloidal flux, or (iii) the geometry of the boundary changes.
!latex This information is required for an efficient iterative scheme that adjusts the boundary of each domain in order to obtain the global plasma equilibrium.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine method ! this is just a dummy routine; the listed variables are required for the macros;

  use fileunits, only : ounit

  use inputlist, only : Wmethod

  use cputiming, only : Tmethod

  use allglobal, only : myid, cpus

  LOCALS

  BEGIN(method)

  RETURN(method)

end subroutine method

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
