!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Solves Beltrami linear system for given helicity multiplier and poloidal flux and returns an error function.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine mp00ac( Nmudp, Xmudp, Fmudp, Dmudp, LDFJAC, iflag ) ! argument list is fixed by NAG; ma02aa calls mp00ac through C05PCF;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, half, one, two, goldmean

  use numerical, only : machprec, small

  use fileunits, only : ounit

  use inputlist, only : Wmp00ac, Wtr00ab, Lgeometry, Nvol, Nofe, Ni, &
                        mu, helicity, iota, Lposdef,                 &
                        Lprecon, sparseits, ssoromega, sparsetol,    &
                        Lconstraint,                                 &
                        Lexamine

  use cputiming, only : Tmp00ac

  use allglobal, only : cpus, myid, ivol, interfacelabel,                    &
                        Lcoordinatesingularity, Louterboundarynormal,        &
                        mn, im, in,                                          &
                        mns,                                                 &
                        NBdof,                                               &
                        doAt, doAz,                                          &
                        doAtmn, doAzmn,                                      &
                        Ldensematrix, Lsparsematrix,                         &
                        iMB, MB, MR, duMB, duMR,                             &
                        Lpreconditioner,                                     &
                        dpflux, diota,                                       &
                        lABintegral, dABintegral,                            &
                        NNZ,                                                 &
                        interfacelabel,                                      &
                        efmn, ofmn, cfmn, sfmn, trigm, trign,                &
                        Ntz, ijreal, ijimag, trigwk, isr, Nt, Nz,            &
                          sg,   gij,                                         &
                        xoffset ,                                            &
                        Lmp00ac
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS

  INTEGER, intent(in)  :: Nmudp, LDFJAC
  REAL   , intent(in)  :: Xmudp(1:Nmudp)
  REAL                 :: Fmudp(1:Nmudp), Dmudp(1:LDFJAC,1:Nmudp)
  INTEGER              :: iflag ! indicates whether (i) iflag=1: ``function'' values are required; or (ii) iflag=2: ``derivative'' values are required;

  INTEGER              :: lvol ! volume index;

  INTEGER              :: lflag

  REAL                 :: lmu, dpf

  REAL   , allocatable :: soln(:,:) ! will contain the soln vector, it's derivatives wrt mu and poloidal flux, and extra storage space for comparison;

! can reorganize below logic so that soln(:,:) --> soln(:);
  
! miscellenous ;
  INTEGER              :: ii, jj, NN, imn, jmn, kmn, Ldo, imp, LNZ, ip, irz, Lcurv, ij, iofree, iocons, ldmupf
  REAL                 :: cpul, sparsevsdenseerror, lss
  REAL   , allocatable :: matrix(:,:), rhs(:), workmatrix(:,:)
  CHARACTER            :: packorunpack

! required for dense solvers ;
  INTEGER              :: if04axf, IA, IBM
  REAL   , allocatable :: workvectora(:), workvectorb(:)
  CHARACTER            :: screen*7
  
! required for f11jef ;
  INTEGER              :: if11jef, Lrwork, itn, maxitn
  INTEGER, allocatable :: irow(:), icol(:), iwork(:)
  REAL                 :: rnorm, omega, tol
  REAL   , allocatable :: lowermatrix(:), rwork(:)
  CHARACTER            :: method*6, precon*1

#ifdef TESTING
!required for f01adf
  INTEGER              :: if01adf, ibdof, jbdof
  REAL                 :: error, timetaken
#endif

  BEGIN(mp00ac)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  FATALMESS(mp00ac,iflag.ne.1.and.iflag.ne.2,invalid iflag)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  lvol = ivol

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{enumerate} \subsubsection{independent parameters controlling Beltrami fields} \begin{enumerate}

!latex \item With the enclosed toroidal flux assumed given, two additional contraints must be provided to completely describe the Beltrami field in the annular volumes.
!latex \begin{enumerate}
!latex \item \verb+Lconstraint=0+ :
!latex the helicity multiplier, $\mu$, and poloidal flux, $\delta \psi_p$, are given
!latex (and then the helicity and the interface rotational-transforms are determined {\em a posteori});
!latex \item \verb+Lconstraint=1+ :
!latex the inner and outer interface rotational-transforms are given
!latex (and then the helicity multiplier, $\mu$, and the enclosed poloidal flux, $\delta \psi_p$, are determined iteratively, and the helicity is determined {\em a posteori});
!latex \item \verb+Lconstraint=2+ :
!latex the enclosed poloidal flux, $\delta \psi_p$, and helicity are given
!latex (and then the helicity multiplier, $\mu$, is determined iteratively, and the interface rotational-transforms are determined {\em a posteori}).
!latex \end{enumerate}

!latex \item This routine typically uses the helicity multiplier, $\mu$, and the enclosed poloidal flux, $\delta \psi_p$, to label the Beltrami field.
!latex If \verb+Lconstraint=1+ or \verb+Lconstraint=2+, the routine \verb+ma02aa+ will iteratively adjust $\mu$ and $\delta \psi_p$ in order to satisfy the required constraints.

!latex \item There are two important exceptions:
!latex \begin{enumerate}
!latex \item the innermost volume, the simple torus, has a different topology to the annular volumes,
!latex and the poloidal flux is not relevant, and so the Beltrami field only depends on $\mu$.
!latex \item if \verb+Lvacuum.gt.0+, then the outermost volume is required to be a vacuum, $\nabla \times {\bf B} = 0$,
!latex and the helicity-multiplier $\mu$ is not a degree of freedom and the Beltrami field only depends on $\delta \psi_p$.
!latex \end{enumerate}

  ;                       lmu = Xmudp(1) - xoffset ! note construction of Xmudp in ma02aa; included so NAG normalizes error to one + mu, rather than mu, where mu may be zero;
  if( Nmudp.eq.2 ) then ; dpf = Xmudp(2) - xoffset
  else                  ; dpf = dpflux(lvol)
  endif

  if( Louterboundarynormal ) then
   ;                      dpf = Xmudp(1) - xoffset ! the poloidal flux is the only degree of freedom; this must be consistent with ma02aa;
   ;                      lmu = mu(lvol)           ! the helicity multiplier is set on input; mu(lvol)=zero for outermost vacuum field;
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  diota(lvol,0:2,0:1) = zero ! rotational-transform, and its derivatives with respect to lmu and dpf, on the inner and outer interface;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  NN = NBdof(lvol) ! shorthand;
  
  RALLOCATE(workvectora,(1:NN))
  RALLOCATE(workvectorb,(1:NN))
  
  if( Lposdef ) then
  else
   RALLOCATE(workmatrix,(1:NN,1:NN)) ! only required if Lposdef = F;
  endif
  
  RALLOCATE(matrix,(1:NN,1:NN))
  RALLOCATE(rhs,(1:NN))
  
  RALLOCATE(soln,(-1:2,1:NN)) ! soln vector and its derivatives wrt helicity multipler and poloidal flux; additional workspace;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  WCALL(mp00ac,ma01ag,( lvol, mn, Nofe, lmu, dpf )) ! returns Beltrami matrices; Beltrami matrix depends on mu & dpflux;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if11jef = 0 ; if04axf = 0 ! error flags;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{enumerate} \subsubsection{exploiting sparse structure} \begin{enumerate}

!latex \item if \verb+Lsparsematrix=T+, then the NAG routine \verb+F11JEF+ is used which exploits the sparse structure of the matrix.

  if( Lsparsematrix .and. .not.Lcoordinatesingularity ) then 

!   if( Lcoordinatesingularity ) then ! compute preconditioner;
!    
!    RALLOCATE(PM,(1:NBdof(lvol)+1,1:NBdof(lvol))) ! preconditioning matrix;
!    
!    PM(1:NN,1:NN) = MB(1:NN,1:NN) ! compute inverse;
!    
!    IA = NN+1 ; if01adf = 1
!    call F01ADF( NN, PM(1:IA,1:NN), IA, if01adf ) ! inverse of real, symmetric, positive-definite matrix; this can serve as preconditioner;
!    
!    do ibdof = 1, NN ; PM(ibdof,1:NN) = PM(ibdof+1,1:NN) 
!    enddo
!    do ibdof = 1, NN ; PM(ibdof,ibdof:NN) = PM(ibdof:NN,ibdof)
!    enddo
!    
!    error = zero     
!    matrix(1:NN,1:NN) = matmul( PM(1:NN,1:NN) , MB(1:NN,1:NN) )
!    do ibdof = 1, NN
!     do jbdof = 1, NN
!      if( ibdof.eq.jbdof ) then ; error = max(error, abs(matrix(ibdof,jbdof)-one) )
!      else                      ; error = max(error, abs(matrix(ibdof,jbdof)    ) )
!      endif
!     enddo
!    enddo
!    
!    cput = MPI_WTIME() ; timetaken = cput - timetaken
!    select case( if01adf ) !                                     1234567890123456
!    case( 0 )    ; if( Wmp00ac ) write(ounit,1034)cput-cpus,myid,lvol,if01adf,"success         ",timetaken,error
!    case( 1 )    ;               write(ounit,1034)cput-cpus,myid,lvol,if01adf,"not +ve definite",timetaken,error
!    case( 2 )    ;               write(ounit,1034)cput-cpus,myid,lvol,if01adf,"input error     ",timetaken,error
!    case default ;               write(ounit,1034)cput-cpus,myid,lvol,if01adf,"illegal ifail   ",timetaken,error
!    end select
!    
!1034 format("mp00ac : "f10.2" ; myid="i3" ; lvol="i3" ; preconditioner ; if01adf="i2" ; "a16" ; time taken = "f9.4" ; inverse error = "es13.5" ;")
!    
!   endif ! end of if( Lcoordinatesingularity ) ;


!latex \item At present, the sparse representation of the matrix is computed from the dense representation, so this is not optimally efficient from a memory perspective.

   cpul = MPI_WTIME() ! start of sparse methods;
   
   if( NNZ(lvol).le.0 ) then ! this is the first calculation of the sparsity structure;
    LNZ = 0
    do imn = 1, NN
     do jmn = 1, imn
      if( iMB(imn,jmn) ) then ; LNZ = LNZ + 1 ! lower triangular;
      endif
     enddo
    enddo
    NNZ(lvol) = LNZ
   endif ! end of if( NNZ(lvol).le.0 ) ;
   
   RALLOCATE(lowermatrix,(1:NNZ(lvol))) ! this is not deallocated?
   IALLOCATE(irow,(1:NNZ(lvol)))
   IALLOCATE(icol,(1:NNZ(lvol)))
   
   matrix(1:NN,1:NN ) = MB(1:NN,1:NN) ! I guess this is to save the original matrix to enable a comparison between sparse and direct solvers;
   
   LNZ = 0
   do imn = 1, NN
    do jmn = 1, imn
     if( iMB(imn,jmn) ) then ; LNZ = LNZ + 1 ; irow(LNZ) = imn ; icol(LNZ) = jmn ; lowermatrix(LNZ) = matrix(imn,jmn) ! lower triangular;
     endif
    enddo
   enddo
   FATALMESS(mp00aa,LNZ.ne.NNZ(lvol),need to count more carefully)
   
   if( Lposdef ) then ; method='CG'     ; Lrwork = 6*NN+120 !latex \item \verb+Lposdef=T+, the conjugate gradient method is used.
   else               ; method='SYMMLQ' ; Lrwork = 7*NN+120 !latex If not, the symmetric Lanczos method is used.
   endif
   
   select case ( Lprecon ) !latex \item Various pre-conditioners are also available: see \verb+Lprecon+ in input.
   case( 0 ) ; precon='N' ; 
   case( 1 ) ; precon='J' ; Lrwork = Lrwork + NN
   case( 2 ) ; precon='S' ; Lrwork = Lrwork + NN
   case default ; FATALMESS(mp00ac,.true., invalid value of Lprecon)
   end select
   
   tol = max( sparsetol, machprec ) ; omega = ssoromega
   
   if( sparseits.le.0 ) then ; maxitn = max(1,abs(sparseits)) * NN
   else                      ; maxitn =           sparseits
   endif
   
   RALLOCATE(rwork,(1:Lrwork))
   RALLOCATE(iwork,(1:NN+1)) ! for iterative solver;
   
   workvectora(1:NN) = MR(1:NN) ! redundant;
   
! imp labels derivative as follows:
! imp = 0 : compute Beltrami field; imp = 1 : compute d Beltrami field / d \mu ; imp = 2 : compute d Beltrami field / d \delta \psi_p ;

   do imp = 0, 2 ! loop over derivatives with respect to helicity multipler and poloidal flux;
    
    if( iflag.eq.1 .and. imp.gt.0 ) cycle ! only need to return function; recall that the derivative estimate requires function evaluation;

    if( iflag.eq.2 ) then ! derivatives of Beltrami field wrt \mu and/or \delta \psi_p are required;
     
     if( Lconstraint.eq.2       .and. imp.eq.2 ) cycle ! enclosed poloidal flux is not a degree of freedom;
     if( Lcoordinatesingularity .and. imp.eq.2 ) cycle ! enclosed poloidal flux is not a degree of freedom;
     if( Louterboundarynormal   .and. imp.eq.1 ) cycle ! helicity multiplier    is not a degree of freedom;

    endif ! end of if( iflag.eq.2 ) ;

   !if( iflag.eq.2 .and. imp.gt.Nmudp ) cycle ! this can only happen if Lconstraint.eq.2, in which case only mu is adjusted to satisfy helicity constraint;
    
   !if( ( Lcoordinatesingularity .or. Louterboundarynormal ) .and. imp.eq.2 ) cycle ! derivatives of innermost/outermost Beltrami field wrt poloidal flux not required;

    if( imp.eq.0 ) rhs(1:NN) =   MR(    1:NN)
    if( imp.eq.1 ) rhs(1:NN) = duMR(imp,1:NN) - matmul( duMB(imp,1:NN,1:NN), soln(0,1:NN) ) ! matrix perturbation;
    if( imp.eq.2 ) rhs(1:NN) = duMR(imp,1:NN)!- matmul( duMB(imp,1:NN,1:NN), soln(0,1:NN) ) ! matrix perturbation; matrix does not depend on poloidal flux;
    
    Ldo = imp ; packorunpack = 'P'
    WCALL(mp00ac,up00aa,( packorunpack, lvol, NN, soln(imp,1:NN), dpf, Ldo )) ! give initial guess;
    
    itn = 0 ; rnorm = zero ; if11jef = 1
    call F11JEF( method, precon, NN, LNZ, lowermatrix(1:LNZ), irow(1:LNZ), icol(1:LNZ), omega, rhs(1:NN), tol, maxitn, soln(imp,1:NN), rnorm, itn, rwork, lrwork, iwork, if11jef )
    
    Ldo = imp
    if( imp.eq.0 ) packorunpack = 'U'
    if( imp.gt.0 ) packorunpack = 'u' ! don't add toroidal and poloidal fluxes to vector potential; recall this is the derivative wrt mu/pflux;

    WCALL(mp00ac,up00aa,( packorunpack, lvol, NN, soln(imp,1:NN), dpf, Ldo )) ! unpacking; this assigns oAt, oAz through common;

!    if( Lexamine ) then
!     rhs(1:NN) = matmul( MB(1:NN,1:NN), soln(imp,1:NN) ) 
!     if( imp.eq.0 ) rhserror = maxval( abs( rhs(1:NN) - (   MR(    1:NN)                                               ) ) )
!     if( imp.eq.1 ) rhserror = maxval( abs( rhs(1:NN) - ( duMR(imp,1:NN) - matmul( duMB(imp,1:NN,1:NN), soln(0,1:NN) ) ) ) )
!     if( imp.eq.2 ) rhserror = maxval( abs( rhs(1:NN) - ( duMR(imp,1:NN)                                               ) ) )
!    else
!     rhserror = zero
!    endif

    cput = MPI_WTIME() !                                                                                            1         2         3         
    select case( if11jef ) !                                                                               1234567890123456789012345678901
    case(  0  )  ; if( Wmp00ac ) write(ounit,1000)cput-cpus,myid,lvol,tol,rnorm,itn,imp,if11jef,cput-cpul,"success                        "
    case(  1,2)  ;               write(ounit,1000)cput-cpus,myid,lvol,tol,rnorm,itn,imp,if11jef,cput-cpul,"input error                    "
    case(  3  )  ;               write(ounit,1000)cput-cpus,myid,lvol,tol,rnorm,itn,imp,if11jef,cput-cpul,"zero diagonal element          "
    case(  4,5)  ;               write(ounit,1000)cput-cpus,myid,lvol,tol,rnorm,itn,imp,if11jef,cput-cpul,"reqd accuracy not obtained     "
    case(  6  )  ;               write(ounit,1000)cput-cpus,myid,lvol,tol,rnorm,itn,imp,if11jef,cput-cpul,"preconditioner not + definite  "
    case(  7  )  ;               write(ounit,1000)cput-cpus,myid,lvol,tol,rnorm,itn,imp,if11jef,cput-cpul,"coefficients not + definite    "
    case(  8  )  ;               write(ounit,1000)cput-cpus,myid,lvol,tol,rnorm,itn,imp,if11jef,cput-cpul,"serious error                  "
    case default ;               write(ounit,1000)cput-cpus,myid,lvol,tol,rnorm,itn,imp,if11jef,cput-cpul,"invalid if11jef                "
    end select
    
   enddo ! end of do imp;
   
   DEALLOCATE(rwork)
   DEALLOCATE(iwork)
   
   if( Ldensematrix ) soln(-1,1:NN) = soln(0,1:NN) ! save copy for comparison with soln constructed assuming dense matrix below;
    
  endif ! end of if( Ldense.eq.0 .or. Ldense.eq.2 ) ;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

1000 format("mp00ac : "f10.2" : myid="i3" ; lvol="i3" ; sparse solver: tol="es9.2"; rnorm="es9.2"; itn="i9"; imp="i2"; if11jef="i3"; time="f9.4"; "a31" ;")
1010 format("mp00ac : "f10.2" : myid="i3" ; lvol="i3" ; dense  solver:     "9x   "        "9x   "      "9x"; imp="i2"; "a7   "="i3"; time="f9.4"; "a31" ;":" error="es9.2" ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{enumerate} \subsubsection{linear solver for dense matrix} \begin{enumerate}
  
  if( Ldensematrix .or. Lcoordinatesingularity ) then !latex \item if \verb+Ldensematrix=T+, then use methods that assume the Beltrami linear matrix is dense.

   cpul = MPI_WTIME()
   
! imp labels derivative as follows:
! imp = 0 : compute Beltrami field; imp = 1 : compute d Beltrami field / d \mu ; imp = 2 : compute d Beltrami field / d \delta \psi_p ;
   
   do imp = 0, 2 ! loop over derivatives with respect to helicity multipler and poloidal flux;
    
    if( iflag.eq.1 .and. imp.gt.0 ) cycle ! only need to return function; recall the derivative estimate requires function evaluation;

    if( iflag.eq.2 ) then ! derivatives of Beltrami field wrt \mu and/or \delta \psi_p are required;
     
     if( Lconstraint.eq.2       .and. imp.eq.2 ) cycle ! enclosed poloidal flux is not a degree of freedom;
     if( Lcoordinatesingularity .and. imp.eq.2 ) cycle ! enclosed poloidal flux is not a degree of freedom;
     if( Louterboundarynormal   .and. imp.eq.1 ) cycle ! helicity multiplier    is not a degree of freedom;

    endif ! end of if( iflag.eq.2 ) ;

    if( imp.eq.0 ) rhs(1:NN) =   MR(    1:NN)
    if( imp.eq.1 ) rhs(1:NN) = duMR(imp,1:NN) - matmul( duMB(imp,1:NN,1:NN), soln(0,1:NN) ) ! matrix perturbation;
    if( imp.eq.2 ) rhs(1:NN) = duMR(imp,1:NN)!- matmul( duMB(imp,1:NN,1:NN), soln(0,1:NN) ) ! matrix perturbation; perturbed matrix is trivial; 
    
    matrix(1:NN,1:NN ) = MB(1:NN,1:NN)

    if( Lposdef ) then !latex \item if \verb+Lposdef=T+, the soln is provided by \verb+F04ASF+, which assumes the matrix is symmetric positive-definite;

     if04axf = 1 ; IA = NN            ; screen="if04asf"
     call F04ASF( matrix(1:NN,1:NN), IA, rhs(1:NN), NN, soln(imp,1:NN), workvectora(1:NN), workvectorb(1:NN), if04axf )
     
    else ! matches if( Lposdef ) ; !latex \item if \verb+Lposdef=F+, the soln is provided by \verb+F04ATF+;
     
     if04axf = 1 ; IA = NN ; IBM = NN ; screen="if04atf"
     call F04ATF( matrix(1:NN,1:NN), IA, rhs(1:NN), NN, soln(imp,1:NN), workmatrix(1:IBM,1:NN), IBM, workvectora(1:NN), workvectorb(1:NN), if04axf )
     
    endif
    
    Ldo = imp
    if( imp.eq.0 ) packorunpack = 'U' !       add toroidal and poloidal fluxes to vector potential; what about non-zero normal field ?;
    if( imp.gt.0 ) packorunpack = 'u' ! don't add toroidal and poloidal fluxes to vector potential; what about non-zero normal field ?;
    
    WCALL(mp00ac,up00aa,( packorunpack, lvol, NN, soln(imp,1:NN), dpf, Ldo )) ! unpacking; this assigns oAt, oAz through common;

!    if( Lexamine ) then 
!     rhs(1:NN) = matmul( MB(1:NN,1:NN), soln(imp,1:NN) ) ! reconstruct right-hand-side; 
!     if( imp.eq.0 ) rhserror = maxval( abs( rhs(1:NN) - (   MR(    1:NN)                                               ) ) ) ! measure error;
!     if( imp.eq.1 ) rhserror = maxval( abs( rhs(1:NN) - ( duMR(imp,1:NN) - matmul( duMB(imp,1:NN,1:NN), soln(0,1:NN) ) ) ) ) ! measure error;
!     if( imp.eq.2 ) rhserror = maxval( abs( rhs(1:NN) - ( duMR(imp,1:NN)                                               ) ) ) ! measure error;
!    else
!     rhserror = zero
!    endif
    
    if( imp.eq.0 .and. Lsparsematrix ) then ; sparsevsdenseerror = maxval( abs(soln(-1,1:NN)-soln(0,1:NN)) ) ! compare with sparse soln constructed above;
    else                                    ; sparsevsdenseerror = zero
    endif
    
    cput = MPI_WTIME()    
    select case( if04axf ) !                                                                        1234567890123456789012345678901
    case(  0  )  ; if( Wmp00ac ) write(ounit,1010)cput-cpus,myid,lvol,imp,screen,if04axf,cput-cpul,"success                        ",sparsevsdenseerror
    case(  1  )  ;               write(ounit,1010)cput-cpus,myid,lvol,imp,screen,if04axf,cput-cpul,"not +ve definite / singular    "
    case(  2  )  ;               write(ounit,1010)cput-cpus,myid,lvol,imp,screen,if04axf,cput-cpul,"ill conditioned                "
    case(  3  )  ;               write(ounit,1010)cput-cpus,myid,lvol,imp,screen,if04axf,cput-cpul,"input error                    "
    case default ;               write(ounit,1010)cput-cpus,myid,lvol,imp,screen,if04axf,cput-cpul,"invalid if04asf / if04atf      "
    end select
    
   enddo ! end of do imp = 0, Ldmupf;
   
  endif ! end of if( Ldense.eq.1 .or. Ldense.eq.2 ) then ;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  DEALLOCATE(workvectora)
  DEALLOCATE(workvectorb)

  if( Lposdef) then
  else
   DEALLOCATE(workmatrix)
  endif
  
  DEALLOCATE(matrix)
  DEALLOCATE(rhs)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( if11jef.ne.0 .or. if04axf.ne.0 ) then ! failed to construct Beltrami field; don't even attempt to construct straight field line angle; return error flag to ma02aa;

   Lmp00ac(lvol) = .false. ! set error flag;

   DEALLOCATE(soln)

   if( iflag.eq.1 ) Fmudp(1:Nmudp        ) = zero ! provide dummy intent out;
   if( iflag.eq.2 ) Dmudp(1:Nmudp,1:Nmudp) = zero ! provide dummy intent out;
   iflag = -1 ! this value will be returned by C05PCF to ma02aa;

   goto 9999

  endif
  
  Lmp00ac(lvol) = .true. ! set error flag; used in fc02aa;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  DEALLOCATE(soln)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \end{enumerate} \subsubsection{construction of error function} \begin{enumerate}
  
!latex \item This routine returns an ``error-function'', ${\bf F}(\mu,\delta \psi_p)$, defined as follows:
  
  select case( Lconstraint ) !latex \begin{enumerate}
   
  case( 0 ) !latex \item if \verb+Lconstraint.eq.0+,

!latex In this case there are no constraints on the transform or the helicity, and the `dummy' routine is returned:
!latex \be {\bf F}=0.\ee

   if( Wtr00ab ) then
    WCALL(mp00ac,tr00ab,( lvol, Nvol, mns, diota(lvol,0:2,0:1), iflag) ) ! only required if user requests screen output;
   endif
   
   Fmudp(1:Nmudp        ) = zero ! provide dummy intent out; Lconstraint.eq.0 indicates no iterations over mu, dpflux are required;
   Dmudp(1:Nmudp,1:Nmudp) = zero ! provide dummy intent out;                                                                      ;
   
  case( 1 ) !latex \item if \verb+Lconstraint.eq.1+,

!latex \be {\bf F}\left( \mu, \delta\psi_p \right) = 
!latex \left( \iotabar_{inn} - \ds \frac{p_{l-l} + \gamma \, p_{r-l}}{q_{l-l} + \gamma \, q_{r-l}} ,
!latex        \iotabar_{out} - \ds \frac{p_{l  } + \gamma \, p_{r  }}{q_{l  } + \gamma \, q_{r  }}
!latex \right)
!latex \ee
!latex where, after computing the Beltrami field,
!latex the transform on the inner, $\iotabar_{inn}$, and outer, $\iotabar_{out}$, adjacent interfaces is computed by constructing straight-field line coordinates;
!latex and the integers $p_l$, $q_l$, $p_r$ and $q_r$ are given on input, and $\gamma = ( 1 + \sqrt{5} ) / 2$ is the golden mean.
   
!NOTE: if iflag.eq.1, then it is not required to compute the derivatives of the transform wrt mu and dpflux; can save a little time perhaps;
   
   WCALL(mp00ac,tr00ab,( lvol, Nvol, mns, diota(lvol,0:2,0:1), iflag) ) ! given vector potential, computes transform on interfaces;
   
   if( Lcoordinatesingularity ) then ! Nmudp=1;
    if( iflag.eq.1 ) Fmudp(1    ) = diota(lvol,0,  1) - iota(       lvol)
    if( iflag.eq.2 ) Dmudp(1  ,1) = diota(lvol,1,  1)                     ! derivative of outer rotational-transform wrt helicity multiplier
   endif
   
   if( Louterboundarynormal   ) then ! Nmudp=1;
    if( iflag.eq.1 ) Fmudp(1    ) = diota(lvol,0,  0) - iota(lvol-1     )
    if( iflag.eq.2 ) Dmudp(1  ,1) = diota(lvol,2,  0)                     ! derivative of inner rotational-transform wrt enclosed poloidal flux;
   endif
   
   if( Nmudp.eq.2 ) then
    if( iflag.eq.1 ) Fmudp(1:2  ) = diota(lvol,0,0:1) - iota(lvol-1:lvol)
    if( iflag.eq.2 ) Dmudp(1:2,1) = diota(lvol,1,0:1)
    if( iflag.eq.2 ) Dmudp(1:2,2) = diota(lvol,2,0:1)
   endif
   
  case( 2 ) !latex \item if \verb+Lconstraint.eq.2+, 

!latex \be {\bf F}(\mu) = \int_l {\bf A}\cdot{\bf B} \, dv - {\cal K}_l,
!latex \ee
!latex where ${\cal K}_l$ is the helicity given on input.
   
   if( Wtr00ab ) then
    lflag = 1
    WCALL(mp00ac,tr00ab,( lvol, Nvol, mns, diota(lvol,0:2,0:1), lflag) ) ! only required if user requests screen output;
   endif

   if( iflag.eq.1 ) then
    Ldo = 0
    WCALL(mp00ac,fu00aa,( lvol, Ldo )) ! calculate volume integrals of B.B and A.B;
    Fmudp(1  ) = lABintegral(lvol) - helicity(lvol)
   endif
   
   if( iflag.eq.2 ) then
    Ldo = 1
    WCALL(mp00ac,fu00aa,( lvol, Ldo )) ! calculate volume integrals of B.B and A.B;
    Dmudp(1,1) = dABintegral
   endif

  case default
   
   FATALMESS(mp00aa,.true.,invalid Lconstraint)
   
  end select !latex \end{enumerate}
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Wmp00ac ) then

   cput = MPI_WTIME()
   select case( iflag )
   case( 0 )    ; write(ounit,3000)cput-cpus,myid,lvol,lmu,dpf,iflag                        ! this is impossible by above logic;
   case( 1 )    ; write(ounit,3000)cput-cpus,myid,lvol,lmu,dpf,iflag,Fmudp(1:Nmudp)
   case( 2 )    ; write(ounit,3010)cput-cpus,myid,lvol,lmu,dpf,iflag,Dmudp(1:Nmudp,1:Nmudp)
   case default
    FATALMESS(mp00ac,.true.,illegal iflag on entry)
   end select
   
  endif

3000 format("mp00ac : "f10.2" : myid="i3" ; lvol="i3" ; (mu,dp)=("es23.15" ,"es23.15" ) ; iflag="i2" ;":" F="2es23.15" ;")
3010 format("mp00ac : "f10.2" : myid="i3" ; lvol="i3" ; (mu,dp)=("es23.15" ,"es23.15" ) ; iflag="i2" ;":" D="4es23.15" ;")
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(mp00ac)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine mp00ac

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
